cbuffer ConstantBuffer : register( b0 )
{
	float2 LensCenter;
	float2 ScreenCenter;
	float2 Scale;
	float2 ScaleIn;
	float4 HmdWarpParam;
}

//͗p
struct vertexIn
{
	float4 pos : POSITION0;
	float2 tex : TEXCOORD0;
};

//o
struct vertexOut
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
};

vertexOut vs_main(vertexIn IN)
{
	vertexOut OUT;

	OUT.pos = IN.pos;
	OUT.tex = IN.tex;

	return OUT;
}

struct pixcelIn
{
	float4 pos : SV_POSITION;
	float2 tex : TEXCOORD0;
};

Texture2D Texture : register( t0 );
SamplerState Linear : register( s0 );

float2 HmdWarp(float2 in01)
{
	float2 theta = (in01 - LensCenter) * ScaleIn; // Scales to [-1, 1]
	float rSq = theta.x * theta.x + theta.y * theta.y;
	float2 rvector= theta * (HmdWarpParam.x + HmdWarpParam.y * rSq +
				HmdWarpParam.z * rSq * rSq +
				HmdWarpParam.w * rSq * rSq * rSq);
	return LensCenter + Scale * rvector;
}

float4 ps_main(pixcelIn IN) : SV_Target
{
	float2 tc = HmdWarp(IN.tex);
	if (any(clamp(tc, ScreenCenter-float2(0.25,0.5),
			ScreenCenter+float2(0.25, 0.5)) - tc))
		return 0;
	return Texture.Sample(Linear, tc);
}
