cbuffer ConstantBuffer : register( b0 )
{
	matrix World;
	matrix View;
	matrix Projection;
	float4 DCol;
        float3 ACol;
        float SPower;
        float3 SCol;
        float TexStrength;
        float SphStrength;
        float SpaStrength;
        float ZOffset;
}

cbuffer BoneCBuffer : register( b1 )
{
	matrix WBone[1024];
}

//VS͗p
struct vertexIn
{
	float4 pos : POSITION;
	float4 nor : NORMAL;
	float2 tex : TEXCOORD0;
};

//VS͗p(PMD/PMX)
struct PMDXvertexIn
{
	float4 pos : POSITION;
	float4 nor : NORMAL;
	float2 tex : TEXCOORD0;
	uint iBone0 : BLENDINDICES0;
	uint iBone1 : BLENDINDICES1;
	uint iBone2 : BLENDINDICES2;
	uint iBone3 : BLENDINDICES3;
	float fWeight0 : BLENDWEIGHT0;
	float fWeight1 : BLENDWEIGHT1;
	float fWeight2 : BLENDWEIGHT2;
	float fWeight3 : BLENDWEIGHT3;
};

//PS͗p
struct pixcelIn
{
	float4 pos : SV_POSITION;
	float3 nor : NORMAL;
	float2 tex : TEXCOORD0;
	float3 eye : TEXCOORD1;
	float3 ldir : TEXCOORD2;
};

pixcelIn vs_main(vertexIn IN)
{
	pixcelIn OUT;

	OUT.pos = mul( IN.pos, World );
	OUT.pos = mul( OUT.pos, View );
	OUT.eye = -OUT.pos.xyz;
	OUT.pos = mul( OUT.pos, Projection );
        OUT.pos.z -= ZOffset;

	IN.nor.w=0.0f;
	OUT.nor = mul(mul( IN.nor, World ), View ).xyz;

	OUT.ldir= mul(float4(0.0f, 0.0f, -1.0f, 0.0f), View).xyz;

	OUT.tex = IN.tex;

	return OUT;
}

void pmdx_bone_proc(out float4 sk_pos,out float4 sk_nor,
			float4 pos, float4 nor, uint iBone,float fWeight)
{
	matrix mWork;
	float4 vOffset;

	mWork=WBone[iBone];
	vOffset.x=mWork[0][3];
	vOffset.y=mWork[1][3];
	vOffset.z=mWork[2][3];
	vOffset.w=0.0f;
	mWork[0][3]=0.0f;
	mWork[1][3]=0.0f;
	mWork[2][3]=0.0f;
	sk_pos = mul(pos-vOffset,mWork)*fWeight;
	sk_nor = mul(nor,mWork)*fWeight;
}

pixcelIn pmdx_vs_main(PMDXvertexIn IN)
{
	pixcelIn OUT;
	float4 sk_pos0,sk_pos1,sk_pos2,sk_pos3;
	float4 sk_nor0,sk_nor1,sk_nor2,sk_nor3;

	IN.nor.w=0.0f;

	pmdx_bone_proc(sk_pos0,sk_nor0,IN.pos,IN.nor,IN.iBone0,IN.fWeight0);
	pmdx_bone_proc(sk_pos1,sk_nor1,IN.pos,IN.nor,IN.iBone1,IN.fWeight1);
	pmdx_bone_proc(sk_pos2,sk_nor2,IN.pos,IN.nor,IN.iBone2,IN.fWeight2);
	pmdx_bone_proc(sk_pos3,sk_nor3,IN.pos,IN.nor,IN.iBone3,IN.fWeight3);
	OUT.pos=sk_pos0+sk_pos1+sk_pos2+sk_pos3;
	OUT.nor=(sk_nor0+sk_nor1+sk_nor2+sk_nor3).xyz;

	OUT.pos.w=1.0f;

	OUT.pos = mul( OUT.pos, World );
	OUT.pos = mul( OUT.pos, View );
	OUT.eye = -OUT.pos.xyz;
	OUT.pos = mul( OUT.pos, Projection );
        OUT.pos.z -= ZOffset;

	OUT.nor = mul(mul( OUT.nor, World ), View ).xyz;

	OUT.ldir= mul(float4(0.0f, 0.0f, -1.0f, 0.0f), View).xyz;

	OUT.tex = IN.tex;

	return OUT;
}

Texture2D Texture : register( t0 );
Texture2D TexSph : register( t1 );
Texture2D TexSpa : register( t2 );
SamplerState Linear : register( s0 );

float4 ps_main(pixcelIn IN) : SV_Target
{
        float4 texcol;
	float4 outcol;
        float2 tex_cd;
	float3 HV;
	float3 Specular;
	float Diffuse;

	IN.nor=normalize(IN.nor);
	IN.ldir=normalize(IN.ldir);
	HV = normalize(IN.eye + IN.ldir);
	Diffuse = max(0, dot(IN.ldir, IN.nor));
	Specular = pow(max(0.00001f, dot(HV, normalize(IN.nor))), SPower) * SCol;
	outcol.rgb = ACol*(1.0f-Diffuse) + DCol.rgb*Diffuse;

	texcol = Texture.Sample(Linear, IN.tex);  // eNX`̐F
	outcol.rgb = outcol.rgb*(1.0f-TexStrength) + texcol.rgb*TexStrength;
	outcol.w=DCol.w*((1.0f-TexStrength)+texcol.w*TexStrength);
	tex_cd.x=IN.nor.x / 2.0f + 0.5f;
	tex_cd.y=IN.nor.y / 2.0f + 0.5f;
	texcol = TexSph.Sample(Linear, tex_cd);  // XtBAeNX`̐F(sph)
	outcol.rgb = outcol.rgb*(1.0f-SphStrength) + (outcol.rgb*texcol.rgb)*SphStrength;
	texcol = TexSpa.Sample(Linear, tex_cd);  // XtBAeNX`̐F(spa)
	outcol.rgb = outcol.rgb*(1.0f-SpaStrength) + (outcol.rgb+texcol.rgb)*SpaStrength;

	outcol.rgb += Specular;
	outcol.rgb = saturate(outcol.rgb);
	return outcol;
}
